//
//  GlyphsFilterProtocol.h
//  GlyphsCore
//
//  Created by Georg Seifert on 08.05.09.
//  Copyright 2009 schriftgestaltung.de. All rights reserved.
//

@class GSLayer;
@class GSFont;
@class GSDialogController;
@class GSInstance;
@class GSGlyph;
#import <GlyphsCore/GSGlyphViewControllerProtocol.h>

/** The protocol defining the filter plugin interface */
@protocol GlyphsFilter
/** The interface version.*/
@property (nonatomic, readonly) NSUInteger interfaceVersion;
/** The title of the plugin.*/
@property (nonatomic, readonly) NSString *title;
/** The keyEquivalent to select the Filter.*/
@property (nonatomic, readonly) NSString *keyEquivalent;
/** the Controller of type GSEditViewController */
@property (nonatomic, unsafe_unretained) NSViewController<GSGlyphViewControllerProtocol> *controller;

/// Glyphs will call that before the filter is invoked.
- (NSError *)setup;

/** This method will be called if one layer is active in Edit View (the nodes are visible) Pay attention to the selection.

 @param layer   The active Layer
 @param error   if something goes wrong, retunes an NSError by reference.
 
 @return YES if successful, otherwise NO. In the later case the error will hold a explanation.
 */

- (BOOL)runFilterWithLayer:(GSLayer *)layer error:(out NSError **)error;

/// This method will be called if the hole glyph is supposed to be processed.
- (BOOL)runFilterWithLayers:(NSArray *)layers error:(out NSError **)error;

@optional

/// do main initialisations
- (void)loadPlugin;

@property (nonatomic, readonly) NSView *view;

/// is set, if the user chose "run last Filter". Then it should keep the settings from the last run.
@property (nonatomic) BOOL rerun;

/// The title of the filter dilaog.
@property (nonatomic, readonly) NSString *dialogTitle;

/// the returned value will be placed as the title of the "OK" button in the filter dialog.
- (NSString *)actionName;

/// @name Export Filter Support

/// This will be call if the filter should be run from on export of the font. (see: Instance > Custom Properties)
- (BOOL)filterNeedsRemoveOverlap;

/** Is called to process font when called from a 'Filter' custom parameter
 
 @param font      The font to process
 @param arguments A list of arguments
 */
- (void)processFont:(GSFont *)font withArguments:(NSArray *)arguments;

/** Is called to process a layer when to preview a Filter custom parameter
 
 @param layer     The layer to process
 @param arguments A list of arguments
 */
- (void)processLayer:(GSLayer *)layer withArguments:(NSArray *)arguments;

/** Is called to process font when called from a 'PreInterpolationFilter' custom parameter
 @param glyph     The glyph to process
 @param arguments A list of arguments
 */
- (void)processGlyph:(GSGlyph *)glyph withArguments:(NSArray *)arguments;

/** This method will be called if one layer is active in Edit View
 
 This is actually only used for the reomve Overlap filter and might be removed when that one goes away.
 
 @param layer  a Layer
 @param options A dict containing some options. This might be "CheckSelection" : YES/NO.
 @param error   if something goes wrong, retunes an NSError by reference.
 
 @return YES if successful, otherwise NO. In the later case the error will hold a explanation.
 */
- (BOOL)runFilterWithLayer:(GSLayer *)layer options:(NSDictionary *)options error:(out NSError **)error;

/// Implement this to enable the
- (NSString *)customParameterString;

/** Implement it if you need to suply your own window.
 
 @return a GSDialogController that controlles the window
 */
- (GSDialogController *)makeWindowController;

/// do update the window
- (void)setupDialog:(GSDialogController *)dialog;

/**
 Return NO, if the plugin should not be added to the Filter menu. 
 
 If not implemented, it defaults to YES.
 */
- (BOOL)needsMenuEntry;
@end

@protocol GlyphsCallback <NSObject>

- (NSUInteger)callOrder;
/** Is called to process a layer before output
 
 @param layer     The layer to process
 @return if there where any changes, return a copy of layer that is than used for further processing
 */
- (BOOL)processLayer:(GSLayer *)layer extraHandles:(NSMutableArray *)extraHandles error:(out NSError *__autoreleasing *)error;

/** Is called to process a layer before output
 
 @param layer     The layer to process
 @param originalGlyph  the original Glyph with all layers
 @param instance  The instance that is used to interpolate
 @param error   if something goes wrong, retunes an NSError by reference.
 @return NO if there is an error or not
 */
- (BOOL)processLayer:(GSLayer *)layer originalGlyph:(GSGlyph *)originalGlyph withInstance:(GSInstance *)instance error:(out NSError *__autoreleasing *)error;

- (BOOL)interpolateMaster:(GSFontMaster *)master font:(GSFont *)font interpolation:(NSDictionary *)interpolation error:(out NSError *__autoreleasing *)error;

- (BOOL)interpolateLayer:(GSLayer *)layer glyph:(GSGlyph *)glyph interpolation:(NSDictionary *)interpolation error:(out NSError *__autoreleasing *)error;

@end
